@@ -25,7 +25,7 @@ module System.Taffybar
   -- like reserving strut space so that window managers don't put windows over
   -- it).
   --
-  -- | The config file in which you specify the gtk+ widgets to render is just a
+  -- The config file in which you specify the gtk+ widgets to render is just a
   -- Haskell source file which is used to produce a custom executable with the
   -- desired set of widgets. This approach requires that taffybar be installed
   -- as a haskell library (not merely as an executable), and that the ghc
@@ -35,7 +35,7 @@ module System.Taffybar
   -- be provided to taffybar for instantiation and execution.
   --
   -- The following code snippet is a simple example of what a taffybar
-  -- configuration might look like (also see @src/System/Taffybar/Example.hs@):
+  -- configuration might look like (also see "System.Taffybar.Example"):
   --
   -- > {-# LANGUAGE OverloadedStrings #-}
   -- > import System.Taffybar
@@ -53,14 +53,14 @@ module System.Taffybar
   -- >   let cpuCfg = defaultGraphConfig { graphDataColors = [ (0, 1, 0, 1), (1, 0, 1, 0.5)]
   -- >                                   , graphLabel = Just "cpu"
   -- >                                   }
-  -- >       clock = textClockNew defaultClockConfig
+  -- >       clock = textClockNewWith defaultClockConfig
   -- >       cpu = pollingGraphNew cpuCfg 0.5 cpuCallback
   -- >       workspaces = workspacesNew defaultWorkspacesConfig
   -- >       simpleConfig = defaultSimpleTaffyConfig
   -- >                        { startWidgets = [ workspaces ]
   -- >                        , endWidgets = [ sniTrayNew, clock, cpu ]
   -- >                        }
-  -- >   startTaffybar $ simpleTaffybar simpleConfig
+  -- >   simpleTaffybar simpleConfig
   --
   -- This configuration creates a bar with four widgets. On the left is a widget
   -- that shows information about the workspace configuration. The rightmost
@@ -80,7 +80,7 @@ module System.Taffybar
   -- started before starting taffybar.
   --
   -- * If you start your window manager using a graphical login manager like gdm
-  -- or kdm, DBus should be started rautomatically for you.
+  -- or kdm, DBus should be started automatically for you.
   --
   -- * If you start xmonad with a different graphical login manager that does
   -- not start DBus for you automatically, put the line @eval \`dbus-launch
@@ -90,6 +90,20 @@ module System.Taffybar
   --
   -- * If you start xmonad via @startx@ or a similar command, add the
   -- above command to ~\/.xinitrc
+  --
+  -- * System tray compatability
+  --
+  -- | "System.Taffybar.Widget.SNITray" only supports the newer
+  -- StatusNotifierItem (SNI) protocol; older xembed applets will not work.
+  -- AppIndicator is also a valid implementation of SNI.
+  --
+  -- Additionally, this module does not handle recognising new tray applets.
+  -- Instead it is necessary to run status-notifier-watcher from the
+  -- [status-notifier-item](https://github.com/taffybar/status-notifier-item)
+  -- package early on system startup.
+  -- In case this is not possible, the alternative widget
+  -- sniTrayThatStartsWatcherEvenThoughThisIsABadWayToDoIt is available, but
+  -- this may not necessarily be able to pick up everything.
 
   -- * Colors
   --
@@ -127,7 +141,6 @@ import           Paths_taffybar ( getDataDir )
 
 -- | The parameters that are passed to Dyre when taffybar is invoked with
 -- 'dyreTaffybar'.
-taffybarDyreParams :: Dyre.Params TaffybarConfig
 taffybarDyreParams =
   Dyre.defaultParams
   { Dyre.projectName = "taffybar"
